Exception: Ferrum::ScriptTimeoutError
- Defined in:
- lib/ferrum/errors.rb
Overview
Raised when an evaluated script takes too long to return a value.
Instance Method Summary collapse
-
#message ⇒ String
Explains that the evaluated script timed out.
Instance Method Details
#message ⇒ String
Explains that the evaluated script timed out.
64 65 66 |
# File 'lib/ferrum/errors.rb', line 64 def "Timed out waiting for evaluated script to return a value" end |