Exception: Ferrum::TimeoutError

Inherits:
Error
  • Object
show all
Defined in:
lib/ferrum/errors.rb

Overview

Raised when waiting for a response from the browser times out.

Instance Method Summary collapse

Instance Method Details

#messageString

Explains that waiting for a response timed out.



48
49
50
51
52
53
54
# File 'lib/ferrum/errors.rb', line 48

def message
  "Timed out waiting for response. It's possible that this happened " \
    "because something took a very long time (for example a page load " \
    "was slow). If so, setting the :timeout option to a higher value might " \
    "help. If this happened on an internal protocol call instead, try " \
    "raising :protocol_timeout."
end